/**
 *   Java applet: Tic-Tac-Toe game for two players
 *
 *   Georgy Gimel'farb 12.08.1998
 *   (last modified: 15.09.98)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class TTTApplet extends Applet {
  Label lab = new Label("Tic-Tac-Toe", Label.CENTER ); // game label
  Panel pan = new Panel();                             // game board's panel
  Button [] sq = new Button [ 9 ];                     // game buttons (3 x 3)
  Button again = new Button( "New game");              // restart button
  int ply = 2;                                         // initial player's number


  public void start() {
    for ( int p = 0; p < 9; p ++ ) {                        // instantiate 9 game buttons,
      sq[ p ] = new Button( "." );                          // set "no mark" label and the unique
      sq[ p ].setActionCommand( Integer.toString( p ) );    // action command (the button number
      sq[ p ].addActionListener( new ActionListener() {     // in a String form), add the action
        public void actionPerformed( ActionEvent e ){       // listener to handle a game move (if
	  int b = new Integer( e.getActionCommand() ).intValue(); // the button is clicked on) by
	  move( b );                                        // restoring the button number and 
	}                                                   // invoking the move() method
      } );
      pan.add( sq[ p ] );                                   // add the button to the game's board
    }      
  }

  public void init() {
    setBackground( Color.green );                           // set the green colour for the game
    setForeground( Color.yellow );                          // board and yellow for the labels;
    setFont( new Font("SansSerif", Font.BOLD, 60 ) );       // set the font type and size;
    resize( 360, 480);                                      // set the window size;
    setLayout( new BorderLayout() );                        // set the border layout in the window;
    add ( "North",  lab );                                  // add game label and game panel to the
    add ( "Center", pan );                                  // window and set the grid 3 x 3
    pan.setLayout( new GridLayout( 3, 3 ) );                // layout for the panel to place the
                                                            // game buttons
    again.addActionListener( new ActionListener() {         // add listener to handle a restart     
      public void actionPerformed( ActionEvent e ){         // when the user clicked on the button 
	setForeground( Color.yellow );                      // "New game" (sets starting labeling,
	for ( int p = 0; p < 9; p ++ )                      // color, and label of the board
                             sq[ p ].setLabel( "." );
        lab.setText( "Tic-Tac-Toe" ); 
        again.setLabel( "New game");
      }                                               
    } );                                                            
    add ("South", again);
  }

  public void move( int b ) {                     // A single move (when the button b is clicked on):
    if ( !hit() && free() ) {                     // if the game is not over and there exist some
      ply = ( ply == 2 )? 1 : 2;                  // unmarked buttons, toggle the players and mark 
      mark( b );                                  // the button (if it was unmarked); if there is
      if ( hit() ) { setForeground( Color.blue ); // a "three-in-a-line" position, congratulate the
      lab.setText ("Player " + ply + " won!");    // winner. 
      again.setLabel( "Click on here");
      }                                           
    }
    else { 
      if ( free() ) setForeground( Color.magenta ); // Otherwise, change the labels colour and ask
      else          setForeground( Color.red );     // to restart the game
      lab.setText ("Restart game!");
      again.setLabel( "Click on here");  
    }
  }

  // Checks if there are unmarked buttons
  public boolean free() {   
     boolean f = false;
     for( int b = 0; b < 9; b++ ) 
        if  ( sq[ b ].getLabel() == "." ) f = true;
     return f;
  }

  // Checks if there is a "three-in-a-line" position
  public boolean hit() {
    boolean hit = singleHit(0,4,8) | singleHit(2,4,6);
    for(int p = 0; p<3; p++) hit |= singleHit(p,p+3,p+6);
    for(int p = 0; p<9; p+=3) hit |= singleHit(p,p+1,p+2);
    return hit; 
  }  
  
  // Marks the button b if it is unmarked
  public void mark( int b ) {
    if ( sq[ b ].getLabel() == "." ) {
      if ( ply == 1 ) sq[ b ].setLabel( "X" );
      else sq[ b ].setLabel( "O" ); 
    }
  } 

  // Checks if given three buttons have the same mark "X" or "O"
  private boolean singleHit( int b1, int b2, int b3 ) {
    return ( ( sq[ b1 ].getLabel() != "." ) &&
             ( sq[ b1 ].getLabel() == sq[ b2 ].getLabel() ) &&
             ( sq[ b1 ].getLabel() == sq[ b3 ].getLabel() ) );
  } 
}

